@rem = '--*-Perl-*--
@echo off
.\bin\perl -w setup.bat %1 %2 %3 %4 %5 %6 %7 %8 %9
goto endofperl
@rem ';
#!perl -w
#################################################################
#
#   script to set up path and shortcuts to the Perl installation
#   run this bat file from the directory where perl is installed
#
#################################################################

@rem = (); # eliminate warning

use Config;
use Win32::API;
use Win32::Shortcut;
use Win32::Registry;
use strict;
use DSUtil;


my $debug = 0;
my $configUser = 0;  # set up user path and shortcuts instead of shared ones
my $shortcutfoldername = "IndigoPerl";


my $perl_directory = CopyFilesToTarget();
MakeShortcuts($shortcutfoldername, $configUser, $perl_directory);
AddPath("$perl_directory\\bin");
UpdateHttpdConfig($perl_directory);




sub CopyFilesToTarget
{
    # COPY FILES TO TARGET DIRECTORY
    print "This script will install and configure Perl 5.6\n\n";
    print "Press enter to install in current directory, \nor enter installation directory:";
    my $installation_directory = <STDIN>;
    die if (!defined $installation_directory);
    chop $installation_directory;
    if ($installation_directory) {
        system("xcopy /e/i/d/k/r/h . \"$installation_directory\"");
        return $installation_directory;
    } else {
        return $Config{'prefix'};
    }
}




sub MakeShortcuts
{
    my ($shortcutfoldername, $configUser, $perl_directory) = @_;

    my ($key);
    if ($configUser || Win32::IsWin95()) {
        $key = 'HKCU\Software\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders\Programs';
    } else {
        $key = 'HKLM\SOFTWARE\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders\Common Programs';
    }

    my $folderdir = DS::GetReg($key);
    my $folderpath = "$folderdir\\$shortcutfoldername";
    print "Creating shortcut folder $folderpath\n";
    mkdir ($folderpath, 0777);

    print "Making shortcut PerlConsole\n";
    my $LINK=new Win32::Shortcut();
    $LINK->{'Path'}="$perl_directory\\bin\\PerlConsole.exe";
    $LINK->{'Arguments'}="";
    $LINK->{'WorkingDirectory'}=$perl_directory;
    $LINK->{'Description'}="PerlConsole";
    $LINK->{'ShowCmd'}=SW_SHOWNORMAL;
    $LINK->Save("$folderpath\\PerlConsole.lnk");

    print "Making shortcut PerlDocumentation\n";
    $LINK->{'Path'}="$perl_directory\\html\\index.html";
    $LINK->{'Arguments'}="";
    $LINK->{'WorkingDirectory'}="$perl_directory\\html";
    $LINK->{'Description'}="PerlDocumentation";
    $LINK->{'ShowCmd'}=SW_SHOWNORMAL;
    $LINK->Save("$folderpath\\PerlDocumentation.lnk");


    if (1) { # this version assumes there is a colocated Apache server on this machine
    print "Making shortcut Start Apache\n";
    $LINK->{'Path'}="$perl_directory\\bin\\apache.exe";
    my $foo = $perl_directory;
    $foo =~ s/\\/\//g;
    #$LINK->{'Arguments'}="-s -f \"$foo/conf/httpd.conf\"";
    $LINK->{'Arguments'}=" -f \"$foo/conf/httpd.conf\"";
    $LINK->{'WorkingDirectory'}=$perl_directory;
    $LINK->{'Description'}="Start Apache";
    $LINK->{'ShowCmd'}=SW_SHOWMINNOACTIVE;
    $LINK->Save("$folderpath\\Start Apache.lnk");

    print "Making shortcut Stop Apache\n";
    $LINK->{'Path'}="$perl_directory\\bin\\apache.exe";
    $foo = $perl_directory;
    $foo =~ s/\\/\//g;
    $LINK->{'Arguments'}="-f \"$foo/conf/httpd.conf\" -k shutdown";
    $LINK->{'WorkingDirectory'}=$perl_directory;
    $LINK->{'Description'}="Stop Apache";
    $LINK->{'ShowCmd'}=SW_SHOWMINNOACTIVE;
    $LINK->Save("$folderpath\\Stop Apache.lnk");
    }

    print "Making shortcut View website<br>\n";
    open (OUT, ">$perl_directory/localhost.url");
    print OUT "[InternetShortcut]\n";
    print OUT "URL=http://localhost\n";
    close OUT;
    $LINK=new Win32::Shortcut();
    $LINK->{'Path'}="$perl_directory\\localhost.url";
    $LINK->{'Arguments'}="";
    $LINK->{'WorkingDirectory'}="c:\\";
    $LINK->{'Description'}="View website";
    $LINK->{'ShowCmd'}=SW_SHOWNORMAL;
    $LINK->Save("$folderpath\\View website.lnk");

    $LINK->Close();
}



sub AddPath
{
    my ($required_path) = @_;
    $required_path = Win32::GetShortPathName ($required_path);

    # Add required_path to PATH environment if necessary
    my $escaped_path;
    #($escaped_path = $Config{'binexp'}) =~ s/\\/\\\\/g;
    ($escaped_path = $required_path) =~ s/\\/\\\\/g;

    if ($ENV{'PATH'} =~ m/(^|;)$escaped_path(;|$)/i) {
        print "No change to path required\n";
    }
    else {
        print "Adding $required_path to path\n";
        if(Win32::IsWinNT()) {
            my $key;
            if ($configUser) {
                $key = 'HKCU\Environment\path';
            } else {
                $key = 'HKLM\SYSTEM\CurrentControlSet\Control\Session Manager\Environment\path';
            }
            my $path_in_registry = DS::GetReg($key);
            if ($path_in_registry =~ m/(^|;)$escaped_path(;|$)/i) {
                print "No change to path required\n";
            }
            else {
                DS::SetReg($key, "$required_path;$path_in_registry");

                #SendMessageTimeout(HWND_BROADCAST, WM_SETTINGCHANGE, 0, (LPARAM) "Environment", 
                #       SMTO_ABORTIFHUNG, 5000, &dwReturnValue); 
                my $SendMessageTimeout = new Win32::API("user32", "SendMessageTimeout", 
                        ['N','N','N','P','N','N','P'], 'N');
                my $dwResult = pack("L", 0);
                $SendMessageTimeout->Call(0xffff, 0x001A, 0, "Environment", 0x0002, 5000,$dwResult);
                }
        }
        else {
            print "Updating c:\\autoexec.bat\n";
            my $data = DS::ReadFile("C:/autoexec.bat");
            $data =~ s/(.*)(^|\n)(set\s+|)(\s*path\s*=\s*)(.*?)(\n|$)/$1$2$3$4$required_path;$5$6/is;
            DS::WriteFile("C:/autoexec.bat", $data);
            my $newpath = "$required_path;$ENV{'PATH'}";
            print "DBG: Updating master environment block\n" if $debug;
            system("\"$Config{'binexp'}\\winset\" PATH=$newpath");
        }
        print "\n\nYour path has been modified\n";
        print "The new path will take effect when any new window is opened\n";
    }
}



sub UpdateHttpdConfig
{
    my ($perl_directory) = @_;
    # update httpd.conf, relocate path relative to new location
    print "Updating httpd.conf\n";
    my $ServerRoot = $perl_directory;
    my $port = $ARGV[0] || "80";

    $ServerRoot =~ s%\\%\/%g;

    my $configFile = "$ServerRoot/conf/httpd.conf";
    my $data = DS::ReadFile($configFile);

    my ($oldServerRoot) = $data =~ m/^\s*ServerRoot\s+\"(.*?)\"/mi;
    my ($oldPort) = $data =~ m/^Port\s+(\d+)$/mi;
    $oldServerRoot || die "Error: Configuration file $configFile is invalid\n";
    if ($ServerRoot ne $oldServerRoot || $port ne $oldPort || (!-e $configFile)) {
        $oldServerRoot =~ s%\\%\\\\%g;  #JIC
        $data =~ s/$oldServerRoot/$ServerRoot/g;
        $data =~ s/^Port\s+\d+$/Port $port/gm;
        print "Configuration updated ServerRoot $oldServerRoot -> $ServerRoot, Port $oldPort -> $port\n";
        DS::WriteFile($configFile, $data);
    }
}



__END__
:endofperl

